﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

#if (SILVERLIGHT)
using C1.Silverlight.Chart;
#else
using C1.WPF.C1Chart;
#endif

namespace ChartSamples
{
  [System.ComponentModel.Description("Aggregate chart")]
  public partial class AggregateChart : UserControl
  {
    Dictionary<string, Brush> _dict = new Dictionary<string, Brush>();

    public AggregateChart()
    {
      InitializeComponent();

      _dict["red"] = CreateBrush(Colors.Red);
      _dict["blue"] = CreateBrush(Color.FromArgb(255,2,150,252));
      _dict["yellow"] = CreateBrush(Colors.Yellow);
      _dict["white"] = CreateBrush(Colors.White);


      cb.ItemsSource = Utils.GetEnumValues(typeof(Aggregate));
      cb.SelectedIndex = 0;
      cb.SelectionChanged += (s, e) =>
        chart.Aggregate = (Aggregate)cb.SelectedIndex;

      int cnt = 30;
      chart.Data.ItemNames = CreateRandomStrings(cnt,
        new string[] { "blue", "red", "white", "yellow" }); ;
      
      var vals = CreateRandomValues(cnt);

      var ds = new DataSeries() { ValuesSource = vals};

      ds.PlotElementLoaded += (s, e) =>
        {
          PlotElement pe = (PlotElement)s;
          if (_dict.ContainsKey(pe.DataPoint.Name))
            pe.Fill = _dict[pe.DataPoint.Name];
          pe.StrokeThickness = 0;
        };

      chart.Data.Children.Add(ds);
      chart.View.AxisX.AnnoTemplate = chart.Resources["al_tmpl"];
      BarColumnOptions.SetRadiusX(chart, 4);
      BarColumnOptions.SetRadiusY(chart, 4);
      Loaded += new RoutedEventHandler(AggregateChart_Loaded);
    }

    void AggregateChart_Loaded(object sender, RoutedEventArgs e)
    {
        cb.Foreground = new SolidColorBrush(Colors.Black);
    }

    Random rnd = new Random();

    double[] CreateRandomValues(int cnt)
    {
      double[] vals = new double[cnt];
      for (int i = 0; i < cnt; i++)
        vals[i] = rnd.NextDouble() * 100;
      return vals;
    }

    string[] CreateRandomStrings(int cnt, string[] list)
    {
      string[] vals = new string[cnt];
      for (int i = 0; i < cnt; i++)
        vals[i] = list[rnd.Next(0, list.Length)];
      return vals;
    }

    Brush CreateBrush(Color clr)
    {
      return new LinearGradientBrush()
      {
        GradientStops = 
        { 
          //new GradientStop() { Color= Colors.Black,  Offset = 0},
          new GradientStop() { Color= Color.FromArgb(128, clr.R,clr.G,clr.B) , Offset = 0},
          new GradientStop() { Color= clr, Offset = 0.2},
          new GradientStop() { Color= clr, Offset = 0.7},
          new GradientStop() { Color= Color.FromArgb(128, clr.R,clr.G,clr.B) , Offset = 1},
        },
        StartPoint = new Point(0, 0),
        EndPoint = new Point(1, 1),
        //Opacity = 0.8
      };
    }
  }

  public class AxisLabelConverter : IValueConverter
  {
    #region IValueConverter Members

    public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
    {
      if ( value!=null )
        return string.Format("../../Resources/flower_{0}.png", value.ToString());

      return value;
    }

    public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
    {
      throw new NotImplementedException();
    }

    #endregion
  }

}
