﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using C1.Silverlight;
using C1.Silverlight.DataGrid;

namespace C1DataGrid_Demo2010
{
    public partial class CustomColumns : System.Windows.Controls.UserControl
    {
        public CustomColumns()
        {
            InitializeComponent();

            // set items source
            DataContext = Data.GetProducts((product) => product.Element("Image") != null && product.Element("Image").Value != "no_image_available_small.jpg");
        }

        private void grid_AutoGeneratingColumn(object sender, C1.Silverlight.DataGrid.DataGridAutoGeneratingColumnEventArgs e)
        {
            if (e.Property.Name == "Description")
            {
                e.Cancel = true;
            }
            if (e.Property.Name == "ID")
            {
                e.Cancel = true;
            }
            if (e.Property.Name == "Name")
            {
                e.Column = new DataGridMultiLineTextColumn(e.Property);
                e.Column.Header = "Multi-line column";
                e.Column.VerticalAlignment = VerticalAlignment.Top;
            }
            if (e.Property.Name == "ImageUrl")
            {
                e.Cancel = true;
            }
            if (e.Property.Name == "ProductSubcategoryID")
            {
                e.Cancel = true;
            }
            if (e.Property.Name == "Available")
            {
                e.Cancel = true;
            }
            if (e.Property.Name == "ExpirationDate")
            {
                e.Cancel = true;
                //e.Column.Format = "MM/dd/yy HH:mm:ss";
                //(e.Column as DataGridDateTimeColumn).DateFormat = 
                //(e.Column as DataGridDateTimeColumn).TimeFormat = C1TimeEditorFormat.ShortTime;
            }
            if (e.Property.Name == "StandardCost")
            {
                e.Column = new DataGridCustomNumericColumn(e.Property);
                var _txtBlock = new System.Windows.Controls.TextBlock()
                {
                    Text = "Std. Cost (tooltip)",
                    HorizontalAlignment = HorizontalAlignment.Right,
                    VerticalAlignment = VerticalAlignment.Center
                };
                System.Windows.Controls.ToolTipService.SetToolTip(_txtBlock, new System.Windows.Controls.ToolTip() { Content = "It's a custom tooltip." });
                e.Column.Header = _txtBlock;
            }
            if (e.Property.Name == "ProductNumber")
            {
                var column = new DataGridMaskedTextColumn(e.Property);
                column.Mask = "aa-aaaa-aa";
                column.TextMaskFormat = MaskFormat.IncludeLiterals;
                e.Column = column;
            }
            if (e.Property.Name == "ProductModelID")
            {
                var comboCol = new DataGridComboBoxColumn(e.Property);
                comboCol.DisplayMemberPath = "Name";
                comboCol.SelectedValuePath = "ProductModelID";
                comboCol.ItemsSource = Data.GetModels();
                e.Column = comboCol;
            }
            if (e.Property.Name == "Color")
            {
                e.Column = new DataGridColorColumn(e.Property);
            }
            if (e.Property.Name == "GifImage")
            {
                e.Column = new DataGridGifColumn(e.Property);
                e.Column.Header = "Gif";
                e.Column.Width = new DataGridLength(64);
            }
        }

        private void CheckBox_Loaded(object sender, RoutedEventArgs e)
        {
            var chk = (sender as System.Windows.Controls.CheckBox);
            foreach (Product product in grid.ItemsSource)
            {
                product.PropertyChanged += (s, e2) =>
                {
                    if (e2.PropertyName == "Available")
                    {
                        if ((s as Product).Available != chk.IsChecked)
                        {
                            chk.IsChecked = null;
                        }
                    }
                };
            }
        }

        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            var chk = (sender as System.Windows.Controls.CheckBox);
            foreach (Product product in grid.ItemsSource)
            {
                product.Available = chk.IsChecked;
            }
        }

        private void grid_LoadedRowPresenter(object sender, DataGridRowEventArgs e)
        {
            //e.Row.HeaderPresenter.Content = (e.Row.DisplayIndex + 1).ToString();
        }

        private void grid_KeyDown(object sender, KeyEventArgs e)
        {
            MyNavigationStrategy navigationStrategy = new MyNavigationStrategy(grid);
            navigationStrategy.HandleKeyDown(e);
        }
    }
}
