﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml.Linq;
using System.IO;
using C1.C1Zip;

namespace C1_MDSLWeb
{
    /// <summary>
    /// Summary description for DataService
    /// </summary>
    [WebService(Namespace = "http://tempuri.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    [System.Web.Script.Services.ScriptService]
    public class DataService : System.Web.Services.WebService
    {
        [WebMethod]
        public byte[] GetData(string tables, bool compress)
        {
            // create DataSet with connection string
            SmartDataSet ds = GetDataSet();

            // load data into DataSet
            ds.Fill(tables.Split(','));
            //ds.Fill("Categories", "Products");//, "Shippers", "Suppliers");

            // persist to output stream
            MemoryStream ms = new MemoryStream();
            if (compress)
            {
                using (C1ZStreamWriter sw = new C1ZStreamWriter(ms))
                    ds.WriteXml(sw, XmlWriteMode.WriteSchema);
            }
            else
            {
                ds.WriteXml(ms, XmlWriteMode.WriteSchema);
            }

            // done
            return ms.ToArray();
        }

        [WebMethod]
        public string UpdateData(byte[] dtAdded, byte[] dtModified, byte[] dtDeleted)
        {
            try
            {
                UpdateData(dtAdded, DataRowState.Added);
                UpdateData(dtModified, DataRowState.Modified);
                UpdateData(dtDeleted, DataRowState.Deleted);
                return null;
            }
            catch (Exception x)
            {
                return x.Message;
            }
        }
        void UpdateData(byte[] data, DataRowState state)
        {
            // no changes, no work
            if (data == null)
            {
                return;
            }

            // load data into dataset
            SmartDataSet ds = GetDataSet();
            MemoryStream ms = new MemoryStream(data);
            ds.ReadXml(ms);
            ds.AcceptChanges();

            // update row states with changes
            foreach (DataTable dt in ds.Tables)
            {
                foreach (DataRow dr in dt.Rows)
                {
                    switch (state)
                    {
                        case DataRowState.Added:
                            dr.SetAdded();
                            break;
                        case DataRowState.Modified:
                            dr.SetModified();
                            break;
                        case DataRowState.Deleted:
                            dr.Delete();
                            break;
                    }
                }
            }

            // update the database
            ds.Update();
        }

        SmartDataSet GetDataSet()
        {
            // get physical location of the mdb file
            string mdb = Path.Combine(Context.Request.PhysicalApplicationPath, @"App_Data\nwind.mdb");

            // check that the file exists
            if (!File.Exists(mdb))
            {
                string msg = string.Format("Cannot find database file {0}.", mdb);
                throw new FileNotFoundException(msg);
            }

            // make sure file is not read-only (source control often does this...)
            FileAttributes att = File.GetAttributes(mdb);
            if ((att & FileAttributes.ReadOnly) != 0)
            {
                att &= ~FileAttributes.ReadOnly;
                File.SetAttributes(mdb, att);
            }

            // use SmartDataSet helper class
            SmartDataSet dataSet = new SmartDataSet();
            dataSet.ConnectionString = "provider=microsoft.jet.oledb.4.0;data source=" + mdb;
            return dataSet;
        }
    }
}
