﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Data;

namespace TutorialsSilverlight
{
    using C1.Silverlight.Data.RiaServices;
    using C1.Data;
    using C1.LiveLinq;
    using C1.LiveLinq.LiveViews;

    public partial class ClientSideQuerying : UserControl
    {
        // This client scope will be used to load the data.
        private RiaClientScope _scope = App.ClientCache.CreateScope();
        private ClientView<Web.Product> _seafoodProductsView;
        private View<ProductInfo> _viewProducts;
        
        public ClientSideQuerying()
        {
            InitializeComponent();

            // Define and bind a live view of expensive non-discontinued products ordered by price.
            _viewProducts =
                (from p in _scope.GetItems<Web.Product>("GetProducts")
                 where !p.Discontinued && p.UnitPrice >= 30
                 orderby p.UnitPrice
                 select new ProductInfo
                 {
                     ProductID = p.ProductID,
                     ProductName = p.ProductName,
                     CategoryID = p.CategoryID,
                     CategoryName = p.Category.CategoryName,
                     UnitPrice = p.UnitPrice,
                     QuantityPerUnit = p.QuantityPerUnit,
                     UnitsInStock = p.UnitsInStock,
                     UnitsOnOrder = p.UnitsOnOrder
                 });

            dataGrid1.ItemsSource = _viewProducts;

            // Define a view of seafood products. Filtering is performed on the server.
            _seafoodProductsView = _scope.GetItems<Web.Product>("GetProducts").AsFiltered(p => p.CategoryID == 8);

            // Bind the label text to the number of products in the view
            textBlockCount.SetBinding(TextBlock.TextProperty, new Binding("Value") { Source = _viewProducts.LiveCount() });
        }
    
        private void addItem(object sender, RoutedEventArgs e)
        {
            // add a new product
            object newItem = ((System.ComponentModel.IEditableCollectionView)dataGrid1.ItemsSource).AddNew();
            dataGrid1.ScrollIntoView(newItem, dataGrid1.Columns[0]);
        }

        private void deleteItem(object sender, RoutedEventArgs e)
        {
            // delete the current product if it exists
            var item = ((System.ComponentModel.ICollectionView)dataGrid1.ItemsSource).CurrentItem;
            if (item != null)
                ((System.ComponentModel.IEditableCollectionView)dataGrid1.ItemsSource).Remove(item);
        }

        private void raiseSeafood(object sender, RoutedEventArgs e)
        {
            // Increase the price of the seafood products.
            // The data grid contents will be updated automatically.
            foreach (var p in _seafoodProductsView)
                p.UnitPrice *= 1.2m;
        }

        private void cutSeafood(object sender, RoutedEventArgs e)
        {
            // Decrease the price of the seafood products.
            // The data grid contents will be updated automatically.
            foreach (var p in _seafoodProductsView)
                p.UnitPrice /= 1.2m;
        }
    }
}
