﻿Imports System.Windows.Data

Imports C1.Data
Imports C1.Silverlight.Data.RiaServices
Imports C1.LiveLinq
Imports C1.LiveLinq.LiveViews
Imports TutorialsSilverlight.Web

Partial Public Class ClientSideQuerying
    Inherits UserControl

    ' This client scope will be used to load the data.
    Private _scope As RiaClientScope = App.ClientCache.CreateScope()
    Private _seafoodProductsView As ClientView(Of Product)
    Private _viewProducts As View(Of Object)

    Public Sub New()
        InitializeComponent()

        ' Define and bind a live view of expensive non-discontinued products ordered by price.
        _viewProducts =
            (From p In _scope.GetItems(Of Product)("GetProducts")
             Where Not p.Discontinued And p.UnitPrice >= 30
             Order By p.UnitPrice
             Select New With
             {
                 p.ProductID,
                 p.ProductName,
                 p.CategoryID,
                 p.Category.CategoryName,
                 .Supplier = p.Supplier.CompanyName,
                 p.UnitPrice,
                 p.QuantityPerUnit,
                 p.UnitsInStock,
                 p.UnitsOnOrder
             }).AsDynamic() ' AsDynamic() is required for data binding because an anonymous class is used (select new...)

        dataGrid1.ItemsSource = _viewProducts

        ' Define a view of seafood products. Filtering is performed on the server.
        _seafoodProductsView = _scope.GetItems(Of Product)("GetProducts").AsFiltered(Function(p) p.CategoryID.HasValue AndAlso p.CategoryID.Value = 8)

        ' Bind the label text to the number of products in the view
        textBlockCount.SetBinding(TextBlock.TextProperty, New Binding("Value") With {.Source = _viewProducts.LiveCount()})
    End Sub

    Private Sub raiseSeafood_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles raiseSeafood.Click
        ' Increase the price of the seafood products.
        ' The data grid contents will be updated automatically.
        For Each p In _seafoodProductsView
            p.UnitPrice *= 1.2
        Next
    End Sub

    Private Sub cutSeafood_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles cutSeafood.Click
        ' Decrease the price of the seafood products.
        ' The data grid contents will be updated automatically.
        For Each p In _seafoodProductsView
            p.UnitPrice /= 1.2
        Next
    End Sub

    Private Sub addItem_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles addItem.Click
        ' add a new product
        Dim newItem = CType(dataGrid1.ItemsSource, System.ComponentModel.IEditableCollectionView).AddNew()
        dataGrid1.ScrollIntoView(newItem, Nothing)
    End Sub

    Private Sub deleteItem_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles deleteItem.Click
        ' delete the current product if it exists
        Dim item = CType(dataGrid1.ItemsSource, System.ComponentModel.ICollectionView).CurrentItem
        If item IsNot Nothing Then
            CType(dataGrid1.ItemsSource, System.ComponentModel.IEditableCollectionView).Remove(item)
        End If
    End Sub
End Class