﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.IO;
using System.Linq;
using System.Windows.Media;

namespace C1Extended_Demo
{
    public class Person
    {
        [Required]
        [StringLength(int.MaxValue, MinimumLength = 2, ErrorMessage = "{0} must contain at least {2} characters.")]
        [Display(Name = "Name", Description = "2 or more characters")]
        public string Name { get; set; }

        [Required]
        [Range(18, 32, ErrorMessage = "{0} must be between {1} and {2}")]
        [Display(Name = "Age", Description = "Between 18 and 32")]
        public int Age { get; set; }

        [Required]
        [RegularExpression(@"^\(\d{3}\)\d{2}-\d{4}", ErrorMessage = "{0} must match the following format (555)55-5555")]
        [Display(Name = "Phone", Description = "(555)55-5555")]
        public string Phone { get; set; }

        [Required]
        [Display(Name = "Favorite Color", Description = "Standard color")]
        [CustomValidation(typeof(Person), "ValidateFavoriteColor")]
        public Color FavoriteColor { get; set; }

        [Required]
        [CustomValidation(typeof(Person), "ValidatePhoto")]
        [Display(Name = "Photo", Description = "*.jpg file")]
        public FileInfo Photo { get; set; }

        public string Address { get; set; }

        public string Residence { get; set; }

        public string Born { get; set; }

        public static ValidationResult ValidateFavoriteColor(Color value)
        {
            Color[] colors = { Color.FromArgb(0xFF, 0xC0, 0x00, 0x00),
                             Color.FromArgb(0xFF, 0xFF, 0x00, 0x00),
                             Color.FromArgb(0xFF, 0xFF, 0xC0, 0x00),
                             Color.FromArgb(0xFF, 0xFF, 0xFF, 0x00),
                             Color.FromArgb(0xFF, 0x92, 0xD0, 0x50),
                             Color.FromArgb(0xFF, 0x00, 0xB0, 0x50),
                             Color.FromArgb(0xFF, 0x00, 0xB0, 0xF0),
                             Color.FromArgb(0xFF, 0x00, 0x70, 0xC0),
                             Color.FromArgb(0xFF, 0x00, 0x20, 0x60),
                             Color.FromArgb(0xFF, 0x70, 0x30, 0xA0)};
            if (colors.Contains(value))
            {
                return ValidationResult.Success;
            }
            else
            {
                return new ValidationResult("You must select a color from the Standard Colors section");
            }
        }

        public static ValidationResult ValidatePhoto(FileInfo value)
        {
            if (System.IO.Path.GetExtension(value.Name).ToLower() == ".jpg")
            {
                return ValidationResult.Success;
            }
            else
            {
                return new ValidationResult("You must select a *.jpg image");
            }
        }

        private static Random Randomizer = new Random();

        public static List<Person> Generate(int qty)
        {
            var items = new List<Person>();
            for (int i = 0; i < qty; i++)
            {
                items.Add(new Person()
                {
                    Name = Names[Randomizer.Next(Names.Length)] + " " + Surnames[Randomizer.Next(Surnames.Length)],
                    Address = Streets[Randomizer.Next(Streets.Length)] + " " + (1000 + Randomizer.Next(8999)).ToString(),
                    Age = 10 + Randomizer.Next(40),
                    Residence = Countries[Randomizer.Next(Countries.Length)],
                    Born = Countries[Randomizer.Next(Countries.Length)],
                });
            }
            return items;
        }

        private static string[] Streets = new string[] 
        {
            "5th Avenue",
            "10th Avenue",
            "13th Street",
            "Solano García",
            "Fray Bentos",
            "Lois Albert Herrera",
            "Brasil Avenue",
            "Kenedy Avenue",
            "Peter Campbell",
            "Missisipi"
        };

        private static string[] Names = new string[] 
        {
            "Michael",
            "John",
            "Bernardo",
            "Alvaro",
            "Max",
            "Leonard",
            "Noela",
            "Gabriel",
            "Bruno",
            "Rodrigo",
            "Sheela",
            "Chris",
            "Martin",
            "Ben",
            "Alex",
            "Irina",
            "Dave",
            "Patric",
        };

        private static string[] Surnames = new string[] 
        {
            "Johnson",
            "Alvarez",
            "Maxtor",
            "Johansen",
            "Vera",
            "García",
            "Days",
            "Castle",
            "Varela",
            "Smith",
            "Gates",
            "Meredith",
            "Drexler",
            "Beckham",
            "Jobs",
        };

        private static string[] Countries = new string[] 
        {
            "Afghanistan", 
            "Albania",
            "Algeria",
            "Andorra",
            "Angola",
            "Antigua and Barbuda",
            "Argentina",
            "Armenia",
            "Australia",
            "Austria",
            "Azerbaijan",
            "Bahrain",
            "Bangladesh",
            "Barbados",
            "Belarus",
            "Belgium",
            "Belize",
            "Benin",
            "Bhutan",
            "Bolivia",
            "Bosnia and Herzegovina",
            "Botswana",
            "Brazil",
            "Brunei",
            "Bulgaria",
            "Burkina Faso",
            "Burundi",
            "Côte d'Ivoire",
            "Cambodia",
            "Cameroon",
            "Canada",
            "Cape Verde",
            "Central African Republic",
            "Chad",
            "Chile",
            "China",
            "Colombia",
            "Comoros",
            "Congo",
            "Costa Rica",
            "Croatia",
            "Cuba",
            "Cyprus",
            "Czech Republic",
            "Democratic Republic of the Congo",
            "Denmark",
            "Djibouti",
            "Dominica",
            "Dominican Republic",
            "Ecuador",
            "Egypt",
            "El Salvador",
            "Equatorial Guinea",
            "Eritrea",
            "Estonia",
            "Ethiopia",
            "Fiji",
            "Finland",
            "Former Yugoslav Republic of Macedonia",
            "France",
            "Gabon",
            "Georgia",
            "Germany",
            "Ghana",
            "Greece",
            "Grenada",
            "Guatemala",
            "Guinea",
            "Guinea-Bissau",
            "Guyana",
            "Haiti",
            "Honduras",
            "Hungary",
            "Iceland",
            "India",
            "Indonesia",
            "Iran",
            "Iraq",
            "Ireland",
            "Israel",
            "Italy",
            "Jamaica",
            "Japan",
            "Jordan",
            "Kazakhstan",
            "Kenya",
            "Kiribati",
            "Kuwait",
            "Kyrgyzstan",
            "Laos",
            "Latvia",
            "Lebanon",
            "Lesotho",
            "Liberia",
            "Libya",
            "Liechtenstein",
            "Lithuania",
            "Luxembourg ",
            "Macau -- ",
            "Madagascar",
            "Malawi",
            "Malaysia",
            "Maldives",
            "Mali",
            "Malta",
            "Marshall Islands",
            "Mauritania",
            "Mauritius",
            "Mexico",
            "Micronesia",
            "Moldova",
            "Monaco",
            "Mongolia",
            "Morocco",
            "Mozambique",
            "Myanmar",
            "Namibia",
            "Nauru",
            "Nepal",
            "Netherlands",
            "New Zealand",
            "Nicaragua",
            "Niger",
            "Nigeria",
            "North Korea",
            "Norway",
            "Oman",
            "Pakistan",
            "Palau",
            "Panama",
            "Papua New Guinea",
            "Paraguay",
            "Peru",
            "Philippines",
            "Poland",
            "Portugal",
            "Qatar",
            "Romania",
            "Russia",
            "Rwanda",
            "São Tomé and Príncipe",
            "Saint Kitts and Nevis",
            "Saint Lucia",
            "Saint Vincent and the Grenadines",
            "Samoa",
            "San Marino",
            "Saudi Arabia",
            "Senegal",
            "Seychelles",
            "Sierra Leone",
            "Singapore",
            "Slovakia",
            "Slovenia",
            "Solomon Islands",
            "Somalia",
            "South Africa",
            "South Korea",
            "Spain",
            "Sri Lanka",
            "Sudan",
            "Suriname",
            "Swaziland",
            "Sweden",
            "Switzerland",
            "Syria",
            "Taiwan",
            "Tajikistan",
            "Tanzania",
            "Thailand",
            "The Bahamas",
            "The Gambia",
            "Togo",
            "Tonga",
            "Trinidad and Tobago",
            "Tunisia",
            "Turkey",
            "Turkmenistan",
            "Tuvalu",
            "Uganda",
            "Ukraine",
            "United Arab Emirates",
            "United Kingdom",
            "United States",
            "Uruguay",
            "Uzbekistan",
            "Vanuatu",
            "Vatican City",
            "Venezuela",
            "Vietnam",
            "Western Sahara",
            "Yemen",
            "Yugoslavia",
            "Zambia",
            "Zimbabwe"
        };
    }

    public class PeopleCollection : List<Person> { }
}
