﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using C1.Silverlight.Extended;
using System.ComponentModel;

namespace GaugeBuilder
{
    public partial class SizeEditor
        : UserControl, ITypeEditorControl
    {
        private PropertyAttribute _selectedProperty;

        public SizeEditor()
        {
            InitializeComponent();
        }

        #region ITypeEditorControl Members

        public event PropertyChangedEventHandler ValueChanged;

        public ITypeEditorControl Create()
        {
            return new SizeEditor();
        }

        public bool Supports(PropertyAttribute Property)
        {
            return (Property.PropertyInfo.PropertyType == typeof(Size));
        }

        public void Attach(PropertyAttribute property)
        {
            _selectedProperty = property;

            // set initial value
            Size controlDefaultValue = (Size) property.PropertyInfo.GetValue(property.SelectedObject, null);
            numHeight.Value = controlDefaultValue.Height; 
            numWidth.Value = controlDefaultValue.Width;

            numHeight.ValueChanged += OnValueChanged;
            numWidth.ValueChanged += OnValueChanged;
        }

        public void Detach(PropertyAttribute property)
        {
            numHeight.ValueChanged -= OnValueChanged;
            numWidth.ValueChanged -= OnValueChanged;
        }

        private void OnValueChanged(object sender, EventArgs e) 
        {
            var newValue = new Size(numWidth.Value, numHeight.Value);
            _selectedProperty.PropertyInfo.SetValue(_selectedProperty.SelectedObject, newValue, null);

            if (ValueChanged != null)
                ValueChanged(this, new PropertyChangedEventArgs(_selectedProperty.MemberName));
        }

        #endregion
    }
}
