﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Text;

namespace CustomToolbar
{
    public partial class OverrideCommands : UserControl
    {
        public OverrideCommands()
        {
            InitializeComponent();

            var loremIpsum = @"<p>Lorem ipsum dolor sit amet, <b>consectetur adipiscing elit</b>. Aliquam vel elementum urna. Cras quis neque ipsum. Nam iaculis iaculis placerat. Mauris magna augue, euismod in sagittis id, ultricies eu metus. Ut luctus, erat id tempor commodo, velit diam gravida arcu, in molestie arcu velit vel mauris. Etiam nunc urna, pellentesque et lacinia quis, sodales quis odio. Vivamus dictum mauris ac augue varius ac iaculis sapien elementum. Vivamus dolor libero, molestie sit amet varius et, viverra sit amet dui. Quisque id ligula vitae libero faucibus gravida vitae eu lorem. Aenean et ultrices arcu. Sed dapibus pellentesque tortor, sit amet convallis ante venenatis sit amet. Donec orci purus, vulputate quis semper eu, blandit at nisi. Nam sit amet velit leo. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Proin id justo elit, id ultrices est. Nam rhoncus blandit tellus id tempus. Nunc sit amet enim ut tellus ultrices laoreet. Maecenas iaculis elementum ante eu dapibus. Nulla ac massa a quam volutpat blandit. Suspendisse et lacus quis est dapibus rhoncus vel quis lorem.</p>";
            var text = new StringBuilder();
            for (int i = 0; i < 10; ++i) text.Append(loremIpsum);
            reportViewer.PageSize = new Size(815, 1056);
            reportViewer.LoadDocumentFromString(text.ToString());


            // override default commands
            // *************************

            // add a document name to the Save & Print commands
            reportViewer.SaveCommand = new MyCommand((param) => true, (param) => reportViewer.SaveDocument("myName"));
            reportViewer.SaveCommand = new MyCommand((param) => true, (param) => reportViewer.PrintDocument("myName"));

            // swap previous & next page commands behavior
            var tmp = reportViewer.NextPageCommand;
            reportViewer.NextPageCommand = reportViewer.PreviousPageCommand;
            reportViewer.PreviousPageCommand = tmp;

            // swap increase & decrease zoom commands behavior
            tmp = reportViewer.IncreaseZoomCommand;
            reportViewer.IncreaseZoomCommand = reportViewer.DecreaseZoomCommand;
            reportViewer.DecreaseZoomCommand = tmp;

            // swap first & last page commands behavior
            tmp = reportViewer.FirstPageCommand;
            reportViewer.FirstPageCommand = reportViewer.LastPageCommand;
            reportViewer.LastPageCommand = tmp;

            // swap find next & previous text commands behavior
            tmp = reportViewer.FindNextCommand;
            reportViewer.FindNextCommand = reportViewer.FindPreviousCommand;
            reportViewer.FindPreviousCommand = tmp;
        }

        private void alert_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            alert.Visibility = System.Windows.Visibility.Collapsed;
        }
    }


    public class MyCommand 
        : ICommand
    {
        Predicate<object> _canExecute;
        Action<object> _execute;

        public MyCommand(Predicate<object> canExecute, Action<object> execute)
        {
            _canExecute = canExecute;
            _execute = execute;
        }

        public bool CanExecute(object parameter)
        {
            if (_canExecute != null)
                return _canExecute(parameter);
            return _execute != null;
        }

        public void OnCanExecuteChanged()
        {
            if (CanExecuteChanged != null)
            {
                CanExecuteChanged(this, EventArgs.Empty);
            }
        }
        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            if (_execute != null) _execute(parameter);
        }
    }
}
