﻿Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports C1.Silverlight.Schedule
Imports C1.C1Schedule
Imports C1.Silverlight
Imports System.Windows.Printing

Namespace AdvancedIntervalAppointment
	Partial Public Class MainPage
		Inherits UserControl
		Implements IDisposable
		Public Sub New()
			InitializeComponent()

			' initialize palette combo
			paletteBox.SelectedIndex = 0

			Dim app As New Appointment(Date.Today.AddDays(-1), TimeSpan.FromDays(1))
			app.AllDayEvent = True
			app.Private = True
			app.Subject = "Test All-Day Event"
			app.Label = sched1.DataStorage.LabelStorage.Labels(8)
			app.Location = "Room 15"
			sched1.DataStorage.AppointmentStorage.Appointments.Add(app)
			app = New Appointment(Date.Today, TimeSpan.FromDays(1))
			app.AllDayEvent = True
			app.BusyStatus = sched1.DataStorage.StatusStorage.Statuses(StatusTypeEnum.Tentative)
			app.Subject = "Another test All-Day Event"
			app.Label = sched1.DataStorage.LabelStorage.Labels(7)
			app.Location = "Room 22"
			sched1.DataStorage.AppointmentStorage.Appointments.Add(app)
			app = New Appointment()
			app.Duration = TimeSpan.FromMinutes(60)
			app.Subject = "Test Appointment"
			app.Location = "Room 327"
			app.Label = sched1.DataStorage.LabelStorage.Labels(5)
			sched1.DataStorage.AppointmentStorage.Appointments.Add(app)
			app = New Appointment()
			app.Duration = TimeSpan.FromMinutes(20)
			app.Subject = "Another test Appointment"
			app.Location = "Room 327"
			app.BusyStatus = sched1.DataStorage.StatusStorage.Statuses(StatusTypeEnum.OutOfOffice)
			sched1.DataStorage.AppointmentStorage.Appointments.Add(app)
			app = New Appointment(Date.Today.AddDays(-3).AddHours(11), TimeSpan.FromMinutes(60))
			app.Subject = "Silverlight lesson"
			app.Location = "Conference room"
			app.Label = sched1.DataStorage.LabelStorage.Labels(8)
			Dim pattern As RecurrencePattern = app.GetRecurrencePattern()
			pattern.RecurrenceType = RecurrenceTypeEnum.Weekly
			pattern.DayOfWeekMask = WeekDaysEnum.Monday
			pattern.Occurrences = 5
			sched1.DataStorage.AppointmentStorage.Appointments.Add(app)
		End Sub

		Private Sub UserControl_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' update toolbar buttons state
			sched1_StyleChanged(Nothing, Nothing)
		End Sub

		Private Sub btnToday_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			sched1.VisibleDates.BeginUpdate()
			sched1.VisibleDates.Clear()
			sched1.VisibleDates.Add(Date.Today)
			sched1.VisibleDates.EndUpdate()
		End Sub

		Private Sub sched1_LayoutUpdated(ByVal sender As Object, ByVal e As EventArgs)
			' update IntervalAppointmentTemplate for all AppointmentsCoverPanes in the control template
			' (you can use different templates for different views, or for AppointmentsCoverPanes with different orientation)
			If needUpdateIntervalAppointmentTemplate Then
				Dim list As IList(Of DependencyObject) = New List(Of DependencyObject)()
				VTreeHelper.GetChildrenOfType(sched1, GetType(AppointmentsCoverPane), list)
				For Each pane As AppointmentsCoverPane In list
					pane.IntervalAppointmentTemplate = sched1.IntervalAppointmentTemplate
				Next pane
				needUpdateIntervalAppointmentTemplate = False
			End If
		End Sub

		Private needUpdateIntervalAppointmentTemplate As Boolean = False
		Private Sub sched1_StyleChanged(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' update toolbar buttons state according to the current C1Scheduler view
            If sched1.Style Is sched1.WorkingWeekStyle Then
                btnWorkWeek.IsChecked = True
            ElseIf sched1.Style Is sched1.WeekStyle Then
                btnWeek.IsChecked = True
            ElseIf sched1.Style Is sched1.MonthStyle Then
                btnMonth.IsChecked = True
            ElseIf sched1.Style Is sched1.TimeLineStyle Then
                btnTimeLine.IsChecked = True
            Else
                btnDay.IsChecked = True
            End If
            If sched1.Style IsNot sched1.MonthStyle Then
                needUpdateIntervalAppointmentTemplate = True
            End If
		End Sub

		#Region "IDisposable Members"

		Public Sub Dispose() Implements IDisposable.Dispose
			sched1.Dispose()
		End Sub

		#End Region

	End Class

	''' <summary>
	''' The <see cref="CustomAppPresenter"/> class overrides tooltip behavior and adds context menu
	''' </summary>
	<TemplateVisualState(Name := "MouseOver", GroupName := "CommonStates"), TemplateVisualState(Name := "Normal", GroupName := "CommonStates")>
	Public Class CustomAppPresenter
		Inherits IntervalAppointmentPresenter
		Public Sub New()
		End Sub

		Private _tip As ToolTip

		Public Overrides Sub OnApplyTemplate()
			MyBase.OnApplyTemplate()
			' add context menu
			Dim menu As New C1ContextMenu()
			Dim item As New C1MenuItem()
			item.Header = "Open"
			menu.Items.Add(item)
			item = New C1MenuItem()
			item.Header = "Remove"
			menu.Items.Add(item)
			item = New C1MenuItem()
			item.Header = "Print"
			menu.Items.Add(item)
			C1ContextMenuService.SetContextMenu(Me, menu)
			AddHandler menu.ItemClick, AddressOf menu_ItemClick

			_tip = TryCast(ToolTipService.GetToolTip(Me), ToolTip)
			If _tip IsNot Nothing Then
				' remove default tooltip
				ToolTipService.SetToolTip(Me, Nothing)
			End If
			_tip = New ToolTip()
			AddHandler _tip.Opened, AddressOf tip_Opened
			ToolTipService.SetToolTip(Me, _tip)

		End Sub

		Private Sub tip_Opened(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' DataContext here is IntervalAppointment object
			' IntervalAppointment.Appointment property represents the source appointment
			' so you can look at appointment properties here and change tooltip content
			_tip.Content = Appointment.Subject & " " & Appointment.Body & " - it's my custom tooltip"
			_tip.IsOpen = True
		End Sub

		Private Sub menu_ItemClick(ByVal sender As Object, ByVal e As SourcedEventArgs)
			Dim item As C1MenuItem = TryCast(e.Source, C1MenuItem)
			Dim menu As C1Menu = TryCast(sender, C1Menu)

			If (CStr(item.Header)).Equals("Print") Then
				' print appointment
				Dim prt As New PrintDocument()
				AddHandler prt.PrintPage, Sub(s, args)
					' create document content (single page)
					Dim text As New TextBlock()
					text.TextWrapping = System.Windows.TextWrapping.Wrap
					text.Width = 500
					text.Text = Appointment.Start.ToString("f") & " - " & Appointment.End.ToShortTimeString() & vbCrLf & Appointment.Subject & " " & Appointment.Location & vbCrLf & Appointment.Body
					args.PageVisual = text
				End Sub
				prt.Print(If(String.IsNullOrEmpty(Appointment.Subject), "Unnamed appointment", Appointment.Subject))
			ElseIf (CStr(item.Header)).Equals("Remove") Then
				' remove appointment
				Appointment.Delete()
			ElseIf (CStr(item.Header)).Equals("Open") Then
				' show Appointment dialog
				Dim sched As C1Scheduler = TryCast(VTreeHelper.GetParentOfType(Me, GetType(C1Scheduler), False), C1Scheduler)
				If sched IsNot Nothing Then
					sched.EditAppointmentDialog(Appointment)
				End If
			End If
		End Sub

		Protected Overrides Sub OnMouseEnter(ByVal e As MouseEventArgs)
			MyBase.OnMouseEnter(e)
			VisualStateManager.GoToState(Me, "MouseOver", True)
		End Sub

		Protected Overrides Sub OnMouseLeave(ByVal e As MouseEventArgs)
			MyBase.OnMouseLeave(e)
			VisualStateManager.GoToState(Me, "Normal", True)
		End Sub
	End Class

End Namespace
